package fr.aminelouveaau.cartes.service;

import fr.aminelouveaau.cartes.model.Card;
import fr.aminelouveaau.cartes.model.Number;
import fr.aminelouveaau.cartes.model.Shape;
import fr.aminelouveaau.cartes.model.api.SortingResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class GameService {
    private final Logger logger = LoggerFactory.getLogger(GameService.class);


    @Value("${cards.hand.number}")
    private Integer cardHandNumber;

    private List<Shape> shapeOrder;
    private List<Number> numberOrder;

    private Stack<Card> deck = new Stack<>();
    private Comparator<Card> comparator;

    @PostConstruct
    public void init() {
        generateOrders();
        createDeck();
    }

    public List<Shape> getShapeOrder() {
        return shapeOrder;
    }

    public List<Number> getNumberOrder() {
        return numberOrder;
    }

    public List<Card> getDeck() {
        return deck;
    }

    public void createDeck() {
        Arrays.stream(Shape.values()).forEach(shape -> {
            Arrays.stream(Number.values()).forEach(number -> {
                deck.push(new Card(shape, number));
            });
        });
        Collections.shuffle(deck);
    }

    private void generateOrders() {
        shapeOrder = Arrays.asList(Shape.values());
        Collections.shuffle(shapeOrder);
        numberOrder = Arrays.asList(Number.values());
        Collections.shuffle(numberOrder);

        comparator = Comparator.comparing((Card c) -> shapeOrder.indexOf(c.getShape()))
                .thenComparing(c -> numberOrder.indexOf(c.getNumber()));
    }

    public List<Card> cardHand() {
        if (this.deck.size() >= this.cardHandNumber) {
            List<Card> hand = new ArrayList<>();
            for (int i = 0; i < this.cardHandNumber; i++) {
                hand.add(this.deck.pop());
            }
            return hand;
        } else {
            throw new HttpClientErrorException(HttpStatus.TOO_MANY_REQUESTS);
        }
    }

    public SortingResult sortHand(List<Card> hand) {
        List<Card> sortedHand = hand.stream().sorted((o1, o2) -> comparator.compare(o1, o2)).collect(Collectors.toList());
        logger.info("Hand : " + hand);
        logger.info("Sorted hand : " + sortedHand);
        return new SortingResult(
                shapeOrder,
                numberOrder,
                sortedHand
        );
    }
}
